--[[
Copyright (C) GtX (Andy), 2022

Author: GtX | Andy
Date: 09.10.2022
Revision: FS22-01

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No modifications may be made to this script, including conversion to other game versions without written permission from GtX | Andy
Copying or removing any part of this code for external use without written permission from GtX | Andy is prohibited.

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
Ohne schriftliche Genehmigung von GtX | Andy dürfen keine Änderungen an diesem Skript vorgenommen werden, einschließlich der Konvertierung in andere Spielversionen
Das Kopieren oder Entfernen irgendeines Teils dieses Codes zur externen Verwendung ohne schriftliche Genehmigung von GtX | Andy ist verboten.
]]


AnimalScreenMoveToHusbandry = {}
AnimalScreenMoveToHusbandry.MOD_NAME = g_currentModName

local AnimalScreenMoveToHusbandry_mt = Class(AnimalScreenMoveToHusbandry, AnimalScreenBase)

function AnimalScreenMoveToHusbandry.new(husbandry, targetHusbandry, infoText, customMt)
    local self = AnimalScreenBase.new(customMt or AnimalScreenMoveToHusbandry_mt)

    self.husbandry = husbandry
    self.targetHusbandry = targetHusbandry

    self.animalScreenInfoText = infoText
    self.isMoveToHusbandryController = true

    local actionText = g_i18n:getText("ui_moveAnimalsToOtherHusbandry", AnimalScreenMoveToHusbandry.MOD_NAME)

    self.sourceActionText = string.format(actionText, targetHusbandry:getName())
    self.targetActionText = string.format(actionText, husbandry:getName())

    return self
end

function AnimalScreenMoveToHusbandry:initSourceItems()
    self.sourceItems = {}

    local clusters = self.husbandry:getClusters()

    if clusters ~= nil then
        for i, cluster in ipairs(clusters) do
            self.sourceItems[i] = AnimalItemStock.new(cluster)
        end
    end
end

function AnimalScreenMoveToHusbandry:initTargetItems()
    self.targetItems = {}

    local clusters = self.targetHusbandry:getClusters()

    if clusters ~= nil then
        for i, cluster in ipairs(clusters) do
            self.targetItems[i] = AnimalItemStock.new(cluster)
        end
    end
end

function AnimalScreenMoveToHusbandry:getSourceName()
    local name = self.husbandry:getName()

    if name == nil or name == "" then
        name = g_i18n:getText("ui_farm")
    end

    local numOfAnimals = self.husbandry:getNumOfAnimals()
    local maxNumOfAnimals = self.husbandry:getMaxNumOfAnimals()

    return string.format("%s (%d / %d)", name, numOfAnimals, maxNumOfAnimals)
end

function AnimalScreenMoveToHusbandry:getTargetName()
    local name = self.targetHusbandry:getName()

    if name == nil or name == "" then
        name = g_i18n:getText("ui_farm")
    end

    local numOfAnimals = self.targetHusbandry:getNumOfAnimals()
    local maxNumOfAnimals = self.targetHusbandry:getMaxNumOfAnimals()

    return string.format("%s (%d / %d)", name, numOfAnimals, maxNumOfAnimals)
end

function AnimalScreenMoveToHusbandry:getSourceMaxNumAnimals(itemIndex)
    local sourceItem = self.sourceItems[itemIndex]

    return math.min(self:getMaxNumAnimals(), sourceItem:getNumAnimals(), self.targetHusbandry:getNumOfFreeAnimalSlots())
end

function AnimalScreenMoveToHusbandry:getTargetMaxNumAnimals(itemIndex)
    local targetItem = self.targetItems[itemIndex]

    return math.min(self:getMaxNumAnimals(), targetItem:getNumAnimals(), self.husbandry:getNumOfFreeAnimalSlots())
end

function AnimalScreenMoveToHusbandry:getApplySourceConfirmationText(itemIndex, numItems)
    local sourceItem = self.sourceItems[itemIndex]
    local moveText = string.format(g_i18n:getText("shop_doYouWantToMoveAnimalsToFarm"), numItems, sourceItem:getName())

    return string.format("%s\n\n%s", moveText, self.targetHusbandry:getName())
end

function AnimalScreenMoveToHusbandry:getApplyTargetConfirmationText(itemIndex, numItems)
    local targetItem = self.targetItems[itemIndex]
    local moveText = string.format(g_i18n:getText("shop_doYouWantToMoveAnimalsToFarm"), numItems, targetItem:getName())

    return string.format("%s\n\n%s", moveText, self.husbandry:getName())
end

function AnimalScreenMoveToHusbandry:applySource(itemIndex, numItems)
    local clusterId = self.sourceItems[itemIndex]:getClusterId()
    local resultCode = AnimalMoveEvent.validate(self.husbandry, self.targetHusbandry, clusterId, numItems, self.husbandry:getOwnerFarmId())
    local resultText = AnimalScreenMoveToHusbandry.getResultText(resultCode, self.husbandry, self.targetHusbandry, false)

    if resultText ~= nil then
        self.errorCallback(resultText)

        return false
    end

    self.actionTypeCallback(AnimalScreenBase.ACTION_TYPE_SOURCE, g_i18n:getText("shop_moveToFarm"))
    g_messageCenter:subscribe(AnimalMoveEvent, self.onAnimalMovedToTarget, self)
    g_client:getServerConnection():sendEvent(AnimalMoveEvent.new(self.husbandry, self.targetHusbandry, clusterId, numItems))

    return true
end

function AnimalScreenMoveToHusbandry:onAnimalMovedToTarget(resultCode)
    g_messageCenter:unsubscribe(AnimalMoveEvent, self)
    self.actionTypeCallback(AnimalScreenBase.ACTION_TYPE_NONE, nil)

    local resultText, success = AnimalScreenMoveToHusbandry.getResultText(resultCode, self.husbandry, self.targetHusbandry, false)
    self.sourceActionFinished(not success, resultText or "")
end

function AnimalScreenMoveToHusbandry:applyTarget(itemIndex, numItems)
    local clusterId = self.targetItems[itemIndex]:getClusterId()
    local resultCode = AnimalMoveEvent.validate(self.targetHusbandry, self.husbandry, clusterId, numItems, self.targetHusbandry:getOwnerFarmId())
    local resultText = AnimalScreenMoveToHusbandry.getResultText(resultCode, self.husbandry, self.targetHusbandry, true)

    if resultText ~= nil then
        self.errorCallback(resultText)

        return false
    end

    self.actionTypeCallback(AnimalScreenBase.ACTION_TYPE_SOURCE, g_i18n:getText("shop_moveToFarm"))
    g_messageCenter:subscribe(AnimalMoveEvent, self.onAnimalMovedToSource, self)
    g_client:getServerConnection():sendEvent(AnimalMoveEvent.new(self.targetHusbandry, self.husbandry, clusterId, numItems))

    return true
end

function AnimalScreenMoveToHusbandry:onAnimalMovedToSource(resultCode)
    g_messageCenter:unsubscribe(AnimalMoveEvent, self)
    self.actionTypeCallback(AnimalScreenBase.ACTION_TYPE_NONE, nil)

    local resultText, success = AnimalScreenMoveToHusbandry.getResultText(resultCode, self.husbandry, self.targetHusbandry, true)
    self.targetActionFinished(not success, resultText or "")
end

function AnimalScreenMoveToHusbandry:onAnimalsChanged(husbandry, clusters)
    if (husbandry == self.husbandry) or (husbandry == self.targetHusbandry) then
        self:initItems() -- Populate source and target tables
        self.animalsChangedCallback() -- Refresh 'AnimalScreen'
    end
end

function AnimalScreenMoveToHusbandry:getSourcePrice(itemIndex, numItems)
    return false, 0, 0, 0
end

function AnimalScreenMoveToHusbandry:getTargetPrice(itemIndex, numItems)
    return false, 0, 0, 0
end

function AnimalScreenMoveToHusbandry:getMaxNumAnimals()
    return 100 -- Increase transfer amount a little but maintain a reasonable max value
end

function AnimalScreenMoveToHusbandry:getAnimalScreenInfoText()
    return self.animalScreenInfoText or ""
end

function AnimalScreenMoveToHusbandry.getResultText(resultCode, sourceHusbandry, targetHusbandry, targetToSource)
    -- Ignore 'AnimalMoveEvent.MOVE_ERROR_NOT_ENOUGH_MONEY', no money is exchanged anyway and it uses the same ID as 'MOVE_ERROR_SOURCE_OBJECT_DOES_NOT_EXIST'

    if resultCode ~= nil then
        if resultCode == AnimalMoveEvent.MOVE_SUCCESS then
            if targetToSource then
                if sourceHusbandry ~= nil then
                    return string.format("%s\n\n%s", g_i18n:getText("shop_movedToFarm"), sourceHusbandry:getName()), true
                end
            else
                if targetHusbandry ~= nil then
                    return string.format("%s\n\n%s", g_i18n:getText("shop_movedToFarm"), targetHusbandry:getName()), true
                end
            end

            return g_i18n:getText("shop_movedToFarm"), true
        elseif resultCode == AnimalMoveEvent.MOVE_ERROR_NO_PERMISSION then
            return g_i18n:getText("shop_messageNoPermissionToTradeAnimals"), false
        elseif resultCode == AnimalMoveEvent.MOVE_ERROR_SOURCE_OBJECT_DOES_NOT_EXIST then
            if sourceHusbandry ~= nil then
                return string.format("%s\n\n%s", g_i18n:getText("shop_messageHusbandryDoesNotExist"), sourceHusbandry:getName())
            end

            return g_i18n:getText("shop_messageHusbandryDoesNotExist"), false
        elseif resultCode == AnimalMoveEvent.MOVE_ERROR_TARGET_OBJECT_DOES_NOT_EXIST then
            if targetHusbandry ~= nil then
                return string.format("%s\n\n%s", g_i18n:getText("shop_messageHusbandryDoesNotExist"), targetHusbandry:getName())
            end

            return g_i18n:getText("shop_messageHusbandryDoesNotExist"), false
        elseif resultCode == AnimalMoveEvent.MOVE_ERROR_INVALID_CLUSTER then
            return g_i18n:getText("shop_messageInvalidCluster"), false
        elseif resultCode == AnimalMoveEvent.MOVE_ERROR_ANIMAL_NOT_SUPPORTED then
            return g_i18n:getText("shop_messageAnimalTypeNotSupported"), false
        elseif resultCode == AnimalMoveEvent.MOVE_ERROR_NOT_ENOUGH_SPACE then
            if targetToSource then
                if sourceHusbandry ~= nil then
                    return string.format("%s\n\n%s", g_i18n:getText("shop_messageNotEnoughSpaceAnimals"), sourceHusbandry:getName()), true
                end
            else
                if targetHusbandry ~= nil then
                    return string.format("%s\n\n%s", g_i18n:getText("shop_messageNotEnoughSpaceAnimals"), targetHusbandry:getName()), true
                end
            end

            return g_i18n:getText("shop_messageNotEnoughSpaceAnimals"), false
        elseif resultCode == AnimalMoveEvent.MOVE_ERROR_NOT_ENOUGH_ANIMALS then
            if targetToSource then
                if targetHusbandry ~= nil then
                    return string.format("%s\n\n%s", g_i18n:getText("shop_messageNotEnoughAnimals"), targetHusbandry:getName()), true
                end
            else
                if sourceHusbandry ~= nil then
                    return string.format("%s\n\n%s", g_i18n:getText("shop_messageNotEnoughAnimals"), sourceHusbandry:getName()), true
                end
            end

            return g_i18n:getText("shop_messageNotEnoughAnimals"), false
        end
    end

    return nil, false
end
