--[[
Copyright (C) GtX (Andy), 2022

Author: GtX | Andy
Date: 09.10.2022
Revision: FS22-01

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No modifications may be made to this script, including conversion to other game versions without written permission from GtX | Andy
Copying or removing any part of this code for external use without written permission from GtX | Andy is prohibited.

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
Ohne schriftliche Genehmigung von GtX | Andy dürfen keine Änderungen an diesem Skript vorgenommen werden, einschließlich der Konvertierung in andere Spielversionen
Das Kopieren oder Entfernen irgendeines Teils dieses Codes zur externen Verwendung ohne schriftliche Genehmigung von GtX | Andy ist verboten.
]]


local modName = g_currentModName
local modDirectory = g_currentModDirectory

local versionString = "0.0.0.0"
local buildId = 1

local validationFail
local numInserted = 0

local function validateTypes(typeManager)
    if typeManager.typeName == "placeable" and numInserted == 0 then
        local specializationName = string.format("%s.husbandryMoveAnimals", modName)
        local specializationObject = g_placeableSpecializationManager:getSpecializationObjectByName(specializationName)

        if specializationObject ~= nil then
            for typeName, typeEntry in pairs (typeManager:getTypes()) do
                if specializationObject.prerequisitesPresent(typeEntry.specializations) then
                    typeManager:addSpecialization(typeName, specializationName)

                    numInserted = numInserted + 1
                end
            end

            if numInserted > 0 then
                local defaultText = g_i18n:getText("shop_transportationFee")

                local function setAnimalScreenInfoText(screen, text)
                    if screen == nil or screen.infoTotal == nil then
                        return
                    end

                    if screen.infoText == nil then
                        local infoBox = screen.infoTotal.parent

                        if infoBox ~= nil then
                            local element = infoBox.elements[11]

                            -- If another mod adds elements then they should be 12 or higher so order should not change but check anyway
                            if element ~= nil and element:isa(TextElement) and element.profile == "animalDealerInfo" then
                                element.id = element.id or "infoText"
                                element:setText(text or defaultText)

                                screen.infoText = element
                            end
                        end
                    else
                        screen.infoText:setText(text or defaultText)
                    end
                end

                setAnimalScreenInfoText(g_animalScreen, defaultText)

                if g_animalScreen.infoText ~= nil then
                    AnimalScreen.registerActionEvents = Utils.appendedFunction(AnimalScreen.registerActionEvents, function (self)
                        if self.controller ~= nil and self.controller.getAnimalScreenInfoText ~= nil then
                            setAnimalScreenInfoText(self, self.controller:getAnimalScreenInfoText())
                        else
                            setAnimalScreenInfoText(self, defaultText)
                        end
                    end)

                    AnimalScreen.removeActionEvents = Utils.appendedFunction(AnimalScreen.removeActionEvents, function (self)
                        setAnimalScreenInfoText(self, defaultText)
                    end)
                end

                Logging.devInfo("[%s] Inserted 'PlaceableHusbandryMoveAnimals' spec into %d placeable husbandry types", modName, numInserted)
            end
        end
    end
end

local function unloadMapData(typeManager)
    if typeManager.typeName == "placeable" then
        numInserted = 0
    end
end

local function validateMod()
    local mod = g_modManager:getModByName(modName)

    if mod == nil or g_iconGenerator ~= nil then
        return true
    end

    versionString = mod.version or versionString

    if mod.modName == "FS22_MoveHusbandryAnimals" or mod.modName == "FS22_MoveHusbandryAnimals_update" then
        if mod.author ~= nil and #mod.author == 3 then
            return true
        end
    end

    validationFail = {
        startUpdateTime = 2000,

        update = function(self, dt)
            self.startUpdateTime = self.startUpdateTime - dt

            if self.startUpdateTime < 0 then
                local text = string.format(g_i18n:getText("moveHusbandryAnimals_loadError", mod.modName), mod.modName, mod.author or "Unknown")

                if g_dedicatedServerInfo == nil then
                    if not g_gui:getIsGuiVisible() then
                        -- Swapped button texts so 'ESC' now ignores the message to avoid accidental clicking.
                        g_gui:showYesNoDialog({
                            title = string.format("%s - Version %s", mod.title, versionString),
                            text = text,
                            dialogType = DialogElement.TYPE_LOADING,
                            callback = self.openModHubLink,
                            yesText = g_i18n:getText("button_modHubDownload"),
                            noText = g_i18n:getText("button_dismiss")
                        })
                    end
                else
                    print("\n" .. text .. "\n    - https://farming-simulator.com/mods.php?lang=en&country=be&title=fs2022&filter=org&org_id=129652&page=0" .. "\n")
                    self.openModHubLink(false)
                end
            end
        end,

        openModHubLink = function(yes)
            if yes then
                local language = g_languageShort
                local link = "mods.php?lang=en&country=be&title=fs2022&filter=org&org_id=129652&page=0"

                if language == "de" or language == "fr" then
                    link = "mods.php?lang=" .. language .. "&country=be&title=fs2022&filter=org&org_id=129652&page=0"
                end

                openWebFile(link, "")
            end

            removeModEventListener(validationFail)
            validationFail = nil
        end
    }

    addModEventListener(validationFail)

    return false
end

local function init()
    if validateMod() then
        source(modDirectory .. "scripts/AnimalScreenMoveToHusbandry.lua")

        g_placeableSpecializationManager:addSpecialization("husbandryMoveAnimals", "PlaceableHusbandryMoveAnimals", modDirectory .. "scripts/PlaceableHusbandryMoveAnimals.lua", nil)

        TypeManager.validateTypes = Utils.prependedFunction(TypeManager.validateTypes, validateTypes)
        TypeManager.unloadMapData = Utils.appendedFunction(TypeManager.unloadMapData, unloadMapData)
    else
        Logging.devInfo("[%s] Failed to validate mod!", modName)
    end
end

init()
